# -*- coding: utf-8 -*-
# Copyright (c) Polyconseil SAS.
# This code is distributed under the two-clause BSD License.

import os.path

from sentry.utils.runner import run_app, initialize_app


def main():
    here = os.path.dirname(__file__)
    run_app(
        project='autoguard',
        default_config_path=os.path.join(here, 'sentry_conf.py'),
        default_settings='autoguard.sentry_conf',
        settings_envvar='SENTRY_CONF',
        initializer=initialize_app,
    )

if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
# Copyright (c) Polyconseil SAS.
# This code is distributed under the two-clause BSD License.
"""SSO using LDAP to get access rights"""

from __future__ import absolute_import, unicode_literals
import logging
import urlparse

from django.conf import settings
from django.contrib.auth.backends import RemoteUserBackend
from django.contrib.auth.middleware import RemoteUserMiddleware
import ldap
from sentry.models.organization import Organization
from sentry.models.organizationmember import OrganizationMember, OrganizationMemberType


class CustomUserHeaderMiddleware(RemoteUserMiddleware):
    header = 'HTTP_X_AUTH_USER'


class LDAPRemoteUserBackend(RemoteUserBackend):
    @staticmethod
    def ldap_uri_parser(uri):
        parsed = urlparse.urlparse(uri)
        login = parsed.username
        password = parsed.password
        netloc = '{}:{}'.format(parsed.hostname, parsed.port or '').strip(':')

        return (
            urlparse.urlunparse((parsed.scheme, netloc, parsed.path, parsed.params, parsed.query, parsed.fragment)),
            (login, password) if login else None
        )

    def __init__(self):
        self.logger = logging.getLogger('{0}.{1}'.format(__name__, LDAPRemoteUserBackend))
        self.ldap_uri, self.ldap_auth = self.ldap_uri_parser(settings.LDAP['uri'])
        self.user_tpl = settings.LDAP['user_tpl']
        self.member_key = settings.LDAP['member_key']
        self.groups = settings.LDAP['groups']

        self.connection = ldap.initialize(self.ldap_uri)
        ldap.set_option(ldap.OPT_X_TLS_REQUIRE_CERT, ldap.OPT_X_TLS_NEVER)  # connection.set_option does not work ???
        if self.ldap_auth is not None:
            self.connection.simple_bind_s(*self.ldap_auth)

    def _ldap_search(self, base, scope, filterstr='(objectClass=*)', attrlist=None, attrsonly=0):
        try:
            result = self.connection.search_s(base, scope, filterstr=filterstr, attrlist=attrlist, attrsonly=attrsonly)
            return result[0][1]
        except Exception as e:
            self.logger.exception(e)

    def _ldap_user(self, username):
        result = self._ldap_search(
            self.user_tpl.format(username=username),
            ldap.SCOPE_BASE,
            attrlist=[b'mail', b'gecos']
        )
        return {
            'email': result['mail'][0],
            'first_name': result['gecos'][0],
        }

    def _ldap_user_in_group(self, username, group):
        result = self._ldap_search(group, ldap.SCOPE_BASE, attrlist=[self.member_key.encode('UTF-8')])
        members = result[self.member_key]
        user = self.user_tpl.format(username=username)

        return user in members

    def authenticate(self, remote_user):
        if (
            not remote_user or
            not any([self._ldap_user_in_group(remote_user, group) for group in self.groups])
        ):
            return

        user = super(LDAPRemoteUserBackend, self).authenticate(remote_user)
        self._add_to_all_organizations(user)

        return user

    @staticmethod
    def _add_to_all_organizations(user):
        if user is None:
            return

        organizations_to_add = (
            Organization.objects
            .order_by('pk')
            .exclude(members=user)
        )

        for organization in organizations_to_add:
            OrganizationMember.objects.get_or_create(
                organization=organization,
                user=user,
                defaults={'type': OrganizationMemberType.ADMIN}
            )

    def configure_user(self, user):
        # Add email and name to user
        ldap_user_attr = self._ldap_user(user.username)
        for key, value in ldap_user_attr.items():
            setattr(user, key, value)
        user.save()

        return user



# -*- coding: utf-8 -*-
# Copyright (c) Polyconseil SAS.
# This code is distributed under the two-clause BSD License.

from __future__ import unicode_literals
import socket
import urlparse
import warnings


def url_value_converter(value):
    value_to_convert = {
        'True': True,
        'False': False,
    }

    return value_to_convert.get(value, value)


def get_hostname(url):
    parsed = urlparse.urlparse(url)
    return parsed.hostname or socket.getfqdn()


def database_from_url(url):
    db_shortcuts = {
        'postgres': 'django.db.backends.postgresql_psycopg2',
        'sqlite': 'django.db.backends.sqlite3',
    }

    parsed = urlparse.urlparse(url)
    parsed_qs = urlparse.parse_qs(parsed.query)
    if any([len(value) > 1 for value in parsed_qs.values()]):
        warnings.warn("settings: django.db_url: one argument is set more than once, only use first one.")

    return {
        'ENGINE': db_shortcuts.get(parsed.scheme, parsed.scheme),
        'NAME': parsed.path[1:],
        'USER': parsed.username,
        'PASSWORD': parsed.password,
        'HOST': parsed.hostname,
        'PORT': parsed.port,
        'OPTIONS': {
            key: url_value_converter(value[0])
            for key, value in parsed_qs.items()
        }
    }


def mail_from_url(url):
    backend_shortcuts = {
        'smtp': 'django.core.mail.backends.smtp.EmailBackend',
        'console': 'django.core.mail.backends.console.EmailBackend',
    }

    parsed = urlparse.urlparse(url)
    parsed_qs = urlparse.parse_qs(parsed.query)
    config = {
        'backend': backend_shortcuts.get(parsed.scheme, parsed.scheme),
        'user': parsed.username,
        'password': parsed.password,
        'host': parsed.hostname,
        'port': parsed.port,
        'use_tls': False,
        'sender': "no-reply@{}".format(get_hostname(url)),
        'prefix': parsed_qs.get('prefix', ['[AUTOGUARD]'])[0],
    }
    config.update({
        key: url_value_converter(value[0])
        for key, value in urlparse.parse_qs(parsed.query).items()
    })
    return config


def sentry_caches_from_url(url):
    parsed = urlparse.urlparse(url)
    backend = parsed.scheme

    backends = {
        'redis': {
            'sentry': 'sentry.cache.redis.RedisCache',
            'buffer': 'sentry.buffer.redis.RedisBuffer',
            'tsdb': 'sentry.tsdb.redis.RedisTSDB',
            'quota': 'sentry.quotas.redis.RedisQuota',
            'rate_limits': 'sentry.ratelimits.redis.RedisRateLimiter',

            'redis_options': {'hosts': {0: {'host': parsed.hostname, 'port': parsed.port}}},
            'django_caches': {'default': {'BACKEND': 'django.core.cache.backends.locmem.LocMemCache'}},
        },
        'memcached': {
            'sentry': 'sentry.cache.django.DjangoCache',
            'buffer': 'sentry.buffer.base.Buffer',  # TODO: check alternative backends
            'tsdb': 'sentry.tsdb.dummy.DummyTSDB',
            'quota': 'sentry.quotas.Quota',
            'rate_limits': 'sentry.ratelimits.base.RateLimiter',

            'redis_options': {},  # Unused in this case, but avoid KeyError
            'django_caches': {'default': {
                'BACKEND': 'django.core.cache.backends.memcached.MemcachedCache',
                'LOCATION': [parsed.netloc],
            }},
        }
    }

    return backends[backend]


def ldapized_middleware_classes(middleware_classes):
    middleware_list = list(middleware_classes)
    i = middleware_list.index('django.contrib.auth.middleware.AuthenticationMiddleware')
    middleware_list[i+1:i+1] = ['autoguard.ldap.CustomUserHeaderMiddleware']
    return tuple(middleware_list)

def ldapized_authentication_backends(authentication_backends):
    authentication_backends_list = list(authentication_backends)
    return tuple(['autoguard.ldap.LDAPRemoteUserBackend'] + authentication_backends_list)
